import {ApplicationState, FALLBACK_CHARSET, FALLBACK_LANGUAGE, FALLBACK_MEDIA_TYPE, Resource} from '../common';
import http from 'http';
import {createServer, CreateServerParams} from './http/server';
import https from 'https';
import {BackendState} from './common';
import {DataSource} from './data-source';

export interface BackendBuilder<T extends DataSource = DataSource> {
	showTotalItemCountOnGetCollection(b?: boolean): this;
	showTotalItemCountOnCreateItem(b?: boolean): this;
	checksSerializersOnDelete(b?: boolean): this;
	throwsErrorOnDeletingNotFound(b?: boolean): this;
	createHttpServer(serverParams?: CreateServerParams): http.Server | https.Server;
	dataSource?: (resource: Resource) => T;
}

export interface CreateBackendParams {
	app: ApplicationState;
	dataSource: DataSource;
}

export const createBackend = (params: CreateBackendParams) => {
	const backendState: BackendState = {
		app: params.app,
		dataSource: params.dataSource,
		cn: {
			language: FALLBACK_LANGUAGE,
			charset: FALLBACK_CHARSET,
			mediaType: FALLBACK_MEDIA_TYPE,
		},
		showTotalItemCountOnGetCollection: false,
		showTotalItemCountOnCreateItem: false,
		throwsErrorOnDeletingNotFound: false,
		checksSerializersOnDelete: false,
	};

	return {
		showTotalItemCountOnGetCollection(b = true) {
			backendState.showTotalItemCountOnGetCollection = b;
			return this;
		},
		showTotalItemCountOnCreateItem(b = true) {
			backendState.showTotalItemCountOnCreateItem = b;
			return this;
		},
		throwsErrorOnDeletingNotFound(b = true) {
			backendState.throwsErrorOnDeletingNotFound = b;
			return this;
		},
		checksSerializersOnDelete(b = true) {
			backendState.checksSerializersOnDelete = b;
			return this;
		},
		createHttpServer(serverParams = {} as CreateServerParams) {
			return createServer(backendState, serverParams);
		},
	} satisfies BackendBuilder;
};
