import * as v from 'valibot';
import {Resource} from '../common';

type IsCreated = boolean;

type TotalCount = number;

type DeleteResult = unknown;

export interface DataSource<Schema = object, Query = object, ID = string> {
	initialize(): Promise<unknown>;
	getTotalCount?(query?: Query): Promise<TotalCount>;
	getMultiple(query?: Query): Promise<Schema[]>;
	getById(id: ID): Promise<Schema | null>;
	getSingle?(query?: Query): Promise<Schema | null>;
	create(data: Schema): Promise<Schema>;
	delete(id: ID): Promise<DeleteResult>;
	emplace(id: ID, data: Schema): Promise<[Schema, IsCreated]>;
	patch(id: ID, data: Partial<Schema>): Promise<Schema | null>;
	prepareResource(resource: Resource): void;
	newId(): Promise<ID>;
}


export interface ResourceIdConfig<IdSchema extends v.BaseSchema> {
	generationStrategy: GenerationStrategy;
	serialize: (id: unknown) => string;
	deserialize: (id: string) => v.Output<IdSchema>;
	schema: IdSchema;
}

export interface GenerationStrategy {
	(dataSource: DataSource, ...args: unknown[]): Promise<string | number | unknown>;
}
