import {ContentNegotiation} from '../../../../common';
import {RequestDecorator} from '../../../common';
import Negotiator from 'negotiator';

declare module '../../../common' {
	interface RequestContext {
		cn: ContentNegotiation;
	}
}

export const decorateRequestWithContentNegotiation: RequestDecorator = (req) => {
	const negotiator = new Negotiator(req);

	const availableLanguages = Array.from(req.backend.app.languages.values() ?? []);
	const availableCharsets = Array.from(req.backend.app.charsets.values() ?? []);
	const availableMediaTypes = Array.from(req.backend.app.mediaTypes.values() ?? []);

	const languageCandidate = negotiator.language(availableLanguages.map((l) => l.name)) ?? req.backend.cn.language.name;
	const charsetCandidate = negotiator.charset(availableCharsets.map((l) => l.name)) ?? req.backend.cn.charset.name;
	const mediaTypeCandidate = negotiator.mediaType(availableMediaTypes.map((l) => l.name)) ?? req.backend.cn.mediaType.name;

	req.cn = {
		language: req.backend.app.languages.get(languageCandidate) ?? req.backend.cn.language,
		mediaType: req.backend.app.mediaTypes.get(mediaTypeCandidate) ?? req.backend.cn.mediaType,
		charset: req.backend.app.charsets.get(charsetCandidate) ?? req.backend.cn.charset,
	};

	return req;
};
