import {ParamRequestDecorator} from '../../../common';
import {CreateServerParams} from '../../server';
import {decorateRequestWithScheme} from './scheme';
import {decorateRequestWithHost} from './host';
import {decorateRequestWithBasePath} from './base-path';

declare module '../../../common' {
	interface RequestContext {
		rawUrl?: string;
		query: URLSearchParams;
	}
}

export const decorateRequestWithUrl: ParamRequestDecorator<[CreateServerParams]> = (serverParams) => (req) => {
	const isHttps = 'key' in serverParams && 'cert' in serverParams;
	decorateRequestWithScheme(isHttps ? 'https' : 'http')(req);
	decorateRequestWithHost(serverParams.host ?? '127.0.0.1')(req);
	decorateRequestWithBasePath(serverParams.basePath ?? '')(req);
	const basePath = new URL(req.basePath, `${req.scheme}://${req.host}`);
	const parsedUrl = new URL(`${basePath.pathname}/${req.url ?? ''}`, basePath.origin);
	req.rawUrl = req.url;
	req.url = req.url?.slice(basePath.pathname.length) ?? '';
	req.query = parsedUrl.searchParams;

	return req;
};
