import {IncomingMessage} from 'http';

export const getBody = (
	req: IncomingMessage,
) => new Promise<Buffer>((resolve, reject) => {
	let body = Buffer.from('');
	req.on('data', (chunk) => {
		body = Buffer.concat([body, chunk]);
	});
	req.on('end', async () => {
		resolve(body);
	});

	req.on('error', (err) => {
		reject(err);
	})
});

interface LinkMapEntry {
	url: string;
	params: Record<string, string>;
}

export class LinkMap extends Set<LinkMapEntry> {
	toString() {
		const entries = Array.from(this.values());

		return entries.map((e) => {
			const params = Object.entries(e.params);

			return [
				`<${encodeURIComponent(e.url)}>`,
				...params.map(([key, value]) => `${encodeURIComponent(key)}="${encodeURIComponent(value)}"`)
			].join(';')
		}).join(',');
	}
}
