import {
	ApplicationState,
	Charset,
	FALLBACK_CHARSET,
	FALLBACK_LANGUAGE,
	FALLBACK_MEDIA_TYPE,
	Language,
	MediaType,
} from '../common';

export interface ClientState {
	app: ApplicationState;
	mediaType: MediaType;
	charset: Charset;
	language: Language;
}

export interface ClientBuilder {
	language(languageCode: ClientState['language']['name']): this;
	charset(charset: ClientState['charset']['name']): this;
	mediaTyoe(mediaType: ClientState['mediaType']['name']): this;
}

export interface CreateClientParams {
	app: ApplicationState;
}

export const createClient = (params: CreateClientParams) => {
	const clientState: ClientState = {
		app: params.app,
		mediaType: FALLBACK_MEDIA_TYPE,
		charset: FALLBACK_CHARSET,
		language: FALLBACK_LANGUAGE
	};

	return {
		mediaTyoe(mediaTypeName) {
			const mediaType = clientState.app.mediaTypes.get(mediaTypeName);
			clientState.mediaType = mediaType ?? FALLBACK_MEDIA_TYPE;
			return this;
		},
		charset(charsetName) {
			const charset = clientState.app.charsets.get(charsetName);
			clientState.charset = charset ?? FALLBACK_CHARSET;
			return this;
		},
		language(languageCode) {
			const language = clientState.app.languages.get(languageCode);
			clientState.language = language ?? FALLBACK_LANGUAGE;
			return this;
		}
	} satisfies ClientBuilder;
};
